import { Application as KoaApplication, Context as KoaContext, Request as KoaRequest, Response as KoaResponse } from '@eggjs/koa';
import type { MiddlewareFunc as KoaMiddlewareFunc, Next } from '@eggjs/koa';
import { EggConsoleLogger, Logger } from 'egg-logger';
import { RegisterOptions, ResourcesController, EggRouter as Router } from '@eggjs/router';
import type { ReadyFunctionArg } from 'get-ready';
import { BaseContextClass } from './base_context_class.js';
import { Timing } from './utils/timing.js';
import type { Fun } from './utils/index.js';
import { Lifecycle } from './lifecycle.js';
import { EggLoader } from './loader/egg_loader.js';
import { EggAppConfig } from './types.js';
import { type SingletonCreateMethod } from './singleton.js';
export declare const EGG_LOADER: unique symbol;
export interface EggCoreOptions {
    baseDir: string;
    type: 'application' | 'agent';
    plugins?: any;
    serverScope?: string;
    env?: string;
}
export type EggCoreInitOptions = Partial<EggCoreOptions>;
export { KoaRequest, KoaResponse, KoaContext, KoaApplication, Router, };
export type { Next, KoaMiddlewareFunc, };
export declare class Request extends KoaRequest {
    app: EggCore;
    response: Response;
}
export declare class Response extends KoaResponse {
    app: EggCore;
    request: Request;
}
export declare class Context extends KoaContext {
    app: EggCore;
    request: Request;
    response: Response;
    service: BaseContextClass;
}
export type MiddlewareFunc<T = Context> = KoaMiddlewareFunc<T>;
export declare class EggCore extends KoaApplication {
    #private;
    options: EggCoreOptions;
    timing: Timing;
    console: EggConsoleLogger;
    BaseContextClass: typeof BaseContextClass;
    Controller: typeof BaseContextClass;
    Service: typeof BaseContextClass;
    Helper?: typeof BaseContextClass;
    lifecycle: Lifecycle;
    loader: EggLoader;
    /** auto inject on loadService() */
    readonly serviceClasses: Record<string, any>;
    /** auto inject on loadController() */
    readonly controller: Record<string, any>;
    /** auto inject on loadMiddleware() */
    readonly middlewares: Record<string, (opt: any, app: EggCore) => MiddlewareFunc>;
    /**
     * @class
     * @param {Object} options - options
     * @param {String} [options.baseDir=process.cwd()] - the directory of application
     * @param {String} [options.type=application|agent] - whether it's running in app worker or agent worker
     * @param {Object} [options.plugins] - custom plugins
     * @since 1.0.0
     */
    constructor(options?: EggCoreInitOptions);
    get logger(): Logger;
    get coreLogger(): Logger;
    /**
     * create a singleton instance
     * @param {String} name - unique name for singleton
     * @param {Function|AsyncFunction} create - method will be invoked when singleton instance create
     */
    addSingleton(name: string, create: SingletonCreateMethod): void;
    /**
     * override koa's app.use, support generator function
     * @since 1.0.0
     */
    use(fn: MiddlewareFunc): this;
    /**
     * Whether `application` or `agent`
     * @member {String}
     * @since 1.0.0
     */
    get type(): "application" | "agent";
    /**
     * The current directory of application
     * @member {String}
     * @see {@link AppInfo#baseDir}
     * @since 1.0.0
     */
    get baseDir(): string;
    /**
     * Alias to {@link https://npmjs.com/package/depd}
     * @member {Function}
     * @since 1.0.0
     */
    get deprecate(): (message: string) => void;
    /**
     * The name of application
     * @member {String}
     * @see {@link AppInfo#name}
     * @since 1.0.0
     */
    get name(): any;
    /**
     * Retrieve enabled plugins
     * @member {Object}
     * @since 1.0.0
     */
    get plugins(): Record<string, import("./types.js").EggPluginInfo>;
    /**
     * The configuration of application
     * @member {Config}
     * @since 1.0.0
     */
    get config(): EggAppConfig;
    /**
     * Execute scope after loaded and before app start.
     *
     * Notice:
     * This method is now NOT recommended and regarded as a deprecated one,
     * For plugin development, we should use `didLoad` instead.
     * For application development, we should use `willReady` instead.
     *
     * @see https://eggjs.org/en/advanced/loader.html#beforestart
     *
     * @param  {Function} scope function will execute before app start
     * @param {string} [name] scope name, default is empty string
     */
    beforeStart(scope: Fun, name?: string): void;
    /**
     * register an callback function that will be invoked when application is ready.
     * @see https://github.com/node-modules/get-ready
     * @since 1.0.0
     * @example
     * const app = new Application(...);
     * app.ready(err => {
     *   if (err) throw err;
     *   console.log('done');
     * });
     */
    ready(): Promise<void>;
    ready(flagOrFunction: ReadyFunctionArg): void;
    /**
     * If a client starts asynchronously, you can register `readyCallback`,
     * then the application will wait for the callback to ready
     *
     * It will log when the callback is not invoked after 10s
     *
     * Recommend to use {@link EggCore#beforeStart}
     * @since 1.0.0
     *
     * @param {String} name - readyCallback task name
     * @param {object} opts -
     *   - {Number} [timeout=10000] - emit `ready_timeout` when it doesn't finish but reach the timeout
     *   - {Boolean} [isWeakDep=false] - whether it's a weak dependency
     * @return {Function} - a callback
     * @example
     * const done = app.readyCallback('mysql');
     * mysql.ready(done);
     */
    readyCallback(name: string, opts: object): (...args: any[]) => void;
    /**
     * Register a function that will be called when app close.
     *
     * Notice:
     * This method is now NOT recommended directly used,
     * Developers SHOULDN'T use app.beforeClose directly now,
     * but in the form of class to implement beforeClose instead.
     *
     * @see https://eggjs.org/en/advanced/loader.html#beforeclose
     *
     * @param {Function} fn - the function that can be generator function or async function.
     */
    beforeClose(fn: Fun, name?: string): void;
    /**
     * Close all, it will close
     * - callbacks registered by beforeClose
     * - emit `close` event
     * - remove add listeners
     *
     * If error is thrown when it's closing, the promise will reject.
     * It will also reject after following call.
     * @return {Promise} promise
     * @since 1.0.0
     */
    close(): Promise<void>;
    /**
     * get router
     * @member {Router} EggCore#router
     * @since 1.0.0
     */
    get router(): Router;
    /**
     * Alias to {@link Router#url}
     * @param {String} name - Router name
     * @param {Object} params - more parameters
     * @return {String} url
     */
    url(name: string, params?: any): string;
    head(path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    head(name: string, path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    get(path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    get(name: string, path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    put(path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    put(name: string, path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    patch(path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    patch(name: string, path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    post(path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    post(name: string, path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    delete(path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    delete(name: string, path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    del(path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    del(name: string, path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    all(path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    all(name: string, path: string | RegExp | (string | RegExp)[], ...middlewares: (MiddlewareFunc | string)[]): EggCore;
    resources(prefix: string, controller: string | ResourcesController): EggCore;
    resources(prefix: string, middleware: MiddlewareFunc, controller: string | ResourcesController): EggCore;
    resources(name: string, prefix: string, controller: string | ResourcesController): EggCore;
    resources(name: string, prefix: string, middleware: MiddlewareFunc, controller: string | ResourcesController): EggCore;
    redirect(source: string, destination: string, status?: number): this;
    register(path: string | RegExp | (string | RegExp)[], methods: string[], middleware: MiddlewareFunc | MiddlewareFunc[], opts?: RegisterOptions): this;
    get [EGG_LOADER](): typeof EggLoader;
}
